package com.danitheskunk.skunkworks.windows;


import com.danitheskunk.skunkworks.Util;
import com.sun.jna.Pointer;
import com.sun.jna.platform.win32.*;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;

import java.util.ArrayList;
import java.util.List;

public class Window {
	private WinDef.HWND hwnd;

	public Window(WinDef.HWND hwnd) {
		this.hwnd = hwnd;
	}

	public String getClassName() {
		var user32 = User32Extra.INSTANCE;
		char[] chars = new char[1024];

		user32.GetClassName(hwnd, chars, 1024);
		return Util.nullTerminatedCharArrayToString(chars);
	}

	public String getExecutableName() {
		var user32 = User32Extra.INSTANCE;
		var kernel32 = Kernel32.INSTANCE;
		var handle = kernel32.OpenProcess(WinNT.PROCESS_QUERY_LIMITED_INFORMATION, false, getProcessId());
		char[] chars = new char[1024];
		var len = new IntByReference();
		len.setValue(1024);

		kernel32.QueryFullProcessImageName(handle, 0, chars, len);
		return Util.nullTerminatedCharArrayToString(chars);
	}

	private int getStyle() {
		var user32 = User32Extra.INSTANCE;
		return user32.GetWindowLong(hwnd, WinUser.GWL_STYLE);
	}

	private boolean isTitleBarVisible() {
		var tb = getTitleBarInfo();
		return (tb.rgstate[0] & User32Extra.STATE_SYSTEM_INVISIBLE) == 0;
	}

	public String getTitle() {
		var user32 = User32Extra.INSTANCE;
		char[] chars = new char[1024];

		user32.GetWindowText(hwnd, chars, 1024);
		return Util.nullTerminatedCharArrayToString(chars);
	}

	private TITLEBARINFO getTitleBarInfo() {
		var user32 = User32Extra.INSTANCE;
		var tb = new TITLEBARINFO();
		user32.GetTitleBarInfo(hwnd, tb);
		return tb;
	}

	private int getProcessId() {
		var user32 = User32Extra.INSTANCE;
		var ref = new IntByReference();
		user32.GetWindowThreadProcessId(hwnd, ref);
		return ref.getValue();
	}

	public boolean isDwmCloaked() {
		var user32 = User32Extra.INSTANCE;
		var dwm = Dwm.INSTANCE;
		var out = new IntByReference();
		dwm.DwmGetWindowAttribute(hwnd, Dwm.DWMWA_CLOAKED, out.getPointer(), new WinDef.DWORD(WinDef.DWORD.SIZE));
		var val = out.getValue();
		return val != 0;
	}
	public boolean isInAltTabList() {
		var user32 = User32Extra.INSTANCE;
		var walk = user32.GetAncestor(hwnd, WinUser.GA_ROOTOWNER);
		WinDef.HWND test;

		while(!(test = user32.GetLastActivePopup(walk)).equals(test)) {
			if(user32.IsWindowVisible(test))
				break;
			walk = test;
		}
		var window = new Window(hwnd);
		return walk.equals(hwnd) && user32.IsWindowVisible(hwnd) && window.getTitle().length() > 0 && !window.isToolWindow() && !window.isDwmCloaked() && !window.getClassName().equals("Progman");
	}

	public boolean isToolWindow() {
		return (getStyle() & User32Extra.WS_EX_TOOLWINDOW) != 0;
	}

	//static functions
	public static List<Window> getAllVisible() {
		var user32 = User32Extra.INSTANCE;
		var windows = new ArrayList<Window>();

		user32.EnumWindows(new WinUser.WNDENUMPROC() {
			@Override
			public boolean callback(WinDef.HWND hwnd, Pointer pointer) {
				var window = new Window(hwnd);
				//if(user32.IsWindowVisible(hwnd) && user32.GetWindow(hwnd, new WinDef.DWORD(User32.GW_OWNER)) == null) {
				if(window.isInAltTabList()) {
					//if(title.length() > 0) {
					//windows.add(window.getTitle() + "  --  " + window.getClassName());
					windows.add(window);
					//}
				}
				return true;
			}
		}, null);

		return windows;
	}
}
